#ifndef PATHFINDER_H_DEF
#define PATHFINDER_H_DEF

#ifdef __cplusplus
extern "C" {
#endif

#include "pathfinder/mathutil.h"
#include "pathfinder/structs.h"

#include "pathfinder/fit.h"
#include "pathfinder/spline.h"
#include "pathfinder/trajectory.h"

#include "pathfinder/modifiers/tank.h"
#include "pathfinder/modifiers/swerve.h"

#include "pathfinder/followers/encoder.h"
#include "pathfinder/followers/distance.h"

#include "pathfinder/io.h"

#ifdef __cplusplus
}
#endif

#endif